Module: Kotoshu::Algorithms::Capitalization
- Defined in:
- lib/kotoshu/algorithms/capitalization.rb
Overview
Capitalization handling for different languages.
Ported from Spylls (Python) capitalization.py
This module provides capitalization detection and conversion for different language casing rules, including special handling for Turkic and German languages.
Defined Under Namespace
Modules: Type Classes: Casing, GermanCasing, TurkicCasing